#Compile Dll "dllCustomControlServer.dll"    'Here is the code for the custom control contained in dllCustomControlServer.dll.
#Dim All                                     'I'd recommend you not use anything but local variables in the custom control
#Include "win32api.inc"                      'code so that multiple instances of the control can be created.  Otherwise,
%MY_CUSTOM_MESSAGE        =1500              'you'll likely end up with one control's memory messing up another's.
%MY_CUS_MSG_LBUTTON_DOWN  =1505


Type WindowColor                  'A pointer to a variable of this type will be sent to the parent or container window
  lpnmh As NMHDR                  'when a %WM_LBUTTONDOWN message is intercepted by the custom control.  Just for the
  szColor As Asciiz*16            'purpose of exposition, the cbWndExtra bytes will be interogated by a Call To
End Type                          'GetWindowLong() to obtain the RGB color stored there.  This will be sent to Parent.


Sub ccCrackPointer Alias "ccCrackPointer" (Byval lParam As Long, Byref wc As WindowColor) Export
  Local ptrWC As WindowColor Ptr

  ptrWC=lParam                                 'I did all this because Visual Basic doesn't handle pointers too well
  wc.lpnmh.hwndFrom=@ptrWC.lpnmh.hwndFrom      'or at all really as far as I know.  So when that pointer to a WindowColor
  wc.lpnmh.idFrom=@ptrWC.lpnmh.idFrom          'type is received in a WM_NOTIFY message in the Window Procedure in
  wc.lpnmh.code=@ptrWC.lpnmh.code              'Visual Basic, I send it back here to be re-processed and have the data
  wc.szColor=@ptrWC.szColor                    'sent back in a regular Type so VB can deal with it.  Maybe you can come
End Sub                                        'a better idea but this is all I could think of doing.


Function blnInitialize Alias "blnInitialize" () Export As Long
  Function=RegisterControlClass()
End Function


'Window Procedure for the custom control.
Function ControlClassWndProc(ByVal hWnd As Long,ByVal Msg As Long,ByVal wParam As Long,ByVal lParam As Long) As Long
  Local hDC,hNewBrush As Dword
  Local wc As WindowColor
  Local ps As PAINTSTRUCT

  Select Case Msg
    Case %WM_CREATE
      Call SetWindowLong(hWnd,0,RGB(255,255,0))   'Set default start up color to yellow and store RGB value
      Function=0                                  'of yellow in bytes 0 - 3 of allocated cbWndExtra bytes.
      Exit Function
    Case %WM_PAINT                                'When the custom control's window becomes invalid for
      hDC=BeginPaint(hWnd,ps)                     'whatever reason a WM_PAINT will be received and this code
      hNewBrush=CreateSolidBrush(GetWindowLong(hWnd,0))            'will run.  Note that a Brush will be
      'hNewBrush=CreateHatchBrush(%HS_CROSS,GetWindowLong(hWnd,0)) 'created of the color retrieved from
      Call FillRect(hDC,ps.rcPaint,hNewBrush)                      'the controls cbWndExtra bytes.  That is...
      Call DrawText(hDC, "Click Me!",-1,ps.rcPaint,%DT_SINGLELINE Or %DT_CENTER Or %DT_VCENTER)
      Call DeleteObject(hNewBrush)                'what the GetWindowLong(hWnd,0) is doing, i.e., retrieving
      Call EndPaint(hWnd,ps)                      'the RGB value stored in cbWndExtra bytes.
      Function=0
      Exit Function
    Case %WM_LBUTTONDOWN                          'If the particular custom control receives a left button
      wc.lpnmh.hwndFrom=hWnd                      'down message a WindowColor variable is filled out with
      wc.lpnmh.idFrom=GetDlgCtrlID(hWnd)          'such items of information as the control's hWnd, its
      wc.lpnmh.code=%MY_CUS_MSG_LBUTTON_DOWN      'Ctrl ID, and the present color of the control retrieved
      Select Case GetWindowLong(hWnd,0)           'from the cbWndExtra bytes.  Also, the wc.lpnmh.code
        Case RGB(255,0,0)   'red                  'member is set to %MY_CUS_MSG_LBUTTONDOWN to let the
          wc.szColor="Red"                        'host know what happened in the custom control.  Finally,
        Case RGB(0,255,0)   'green                'a pointer to the WindowColor type/struct is SendMessage()'ed
          wc.szColor="Green"                      'back to the host.  The message that the host will actually
        Case RGB(255,255,0) 'yellow               'receive is WM_NOTIFY.  The host will then dereference and
          wc.szColor="Yellow"                     'tear apart the WindowColor type to get at the sent data.
        Case RGB(0,0,255)   'blue
          wc.szColor="Blue"
      End Select
      Call SendMessage(GetParent(hWnd),%WM_NOTIFY,GetDlgCtrlID(hWnd),VarPtr(wc))  'Send info back to parent.
      Function=0
      Exit Function
    Case %MY_CUSTOM_MESSAGE                       'This is the message the host sends to the custom control
      Call SetWindowLong(hWnd,0,wParam)           'to let it know it wants it to do something, i.e., paint
      Call InvalidateRect(hWnd,ByVal %NULL,%TRUE) 'its window some particular color.  The host sends an RGB
      Function=0                                  'value in here through the wParam of the message, and here
      Exit Function                               'that wParam (containing an RGB value) is stored immediately
    Case %WM_SIZE                                 'in the cbWndExtra bytes.  Then an InvalidateRect() call
      Call InvalidateRect(hWnd,ByVal %NULL,%TRUE) 'forces a WM_PAINT where the RGB value is retrieved and
      Function=0                                  'the window painted.
      Exit Function
  End Select

  Function=DefWindowProc(hWnd,Msg,wParam,lParam)
End Function


Function RegisterControlClass() As Word                             'Many knowledgeable folks state that a custom control
  Local windowclass As WndClassEx                                   'should export some kind of Initialize() function the
  Local szClassName As Asciiz*16                                    'calling of which by the host app will cause the control's
                                                                    'class to be registered by a function such as directly
  szClassName="Custom_Control"                                      'at left.  At the risk of being savaged by such
  windowclass.cbSize=SizeOf(windowclass)                            'knowledgeable folks I have not done that here.  When
  windowclass.style=%CS_HREDRAW Or %CS_GLOBALCLASS Or %CS_DBLCLKS   'a dll is loaded its DllMain function is called with the
  windowclass.lpfnWndProc=CodePtr(ControlClassWndProc)              'fwdReason parameter set to DLL_PROCESS_ATTACH.  I use
  windowclass.cbClsExtra=0                                          'this message to register the custom control's class.
  windowclass.cbWndExtra=4  'Four extra bytes to store RGB color.   'The Windows documentation states that problems in
  windowclass.hInstance=GetModuleHandle(ByVal %NULL)                'dll load order can in some situations occur if GDI or
  windowclass.hIcon=%NULL                                           'user functions are called during a dll's DLL_PROCESS_ATTACH
  windowclass.hCursor=LoadCursor(%NULL,ByVal %IDC_ARROW)            'message.  Kernel functions are OK but GDI functions can
  windowclass.hbrBackground=GetStockObject(%WHITE_BRUSH)            'be problematic according to the docs.  RegisterClassEx()
  windowclass.lpszMenuName=%NULL                                    'is a user32 function that returns an ATOM (Atoms are in
  windowclass.lpszClassName=VarPtr(szClassName)                     'kernel), and I have never had problems with this.
  windowclass.hIconSm=%NULL                                         'However, if it causes you any concern simply Export

  Function=RegisterClassEx(windowclass)                             'RegisterControlClass() and call it from WinMain()
End Function                                                        'or WM_CREATE.  In that case it wouldn't be necessary...


Function DllMain(ByVal hInstance As Long,ByVal fwdReason As Long,ByVal lpvReserved As Long) As Long
  Select Case As Long fwdReason
    Case %DLL_PROCESS_ATTACH
      'Called When Dll Is Loaded.
    Case %DLL_PROCESS_DETACH
      'Called When Dll Is Unloaded.
  End Select

  DllMain=1
End Function
